""" 
This module was created to handle classification 
models such as Logistic Regression, Linear Discriminant Analysis, 
Quadratic Discriminant Analysis, Naive Bayes and Support Vector Machine.

"""


class LogisticRegression:
    pass

    
class LinearDiscriminantAnalysis:
    pass


class QuadraticDiscriminantAnalysis:
    pass


class NaiveBayes:
    pass


class SupportVectorMachines:
    
    def one_versus_one():
        """Classification using SVM with more than two classes.
        """
        pass
    
    def one_versus_all  ():
        """Classification using SVM with more than two classes.
        """
        pass



""" 
This module was created to handle some 
of the Dimension Reduction methods.

"""


def pca():
    """Principal Components Analysis
    """
    pass


def pcr():
    """Principal Components Regression
    """
    pass


def partial_least_square():
    pass


""" 
This module was created to handle non-linear
models such as Bagging, Random Forests, Boosting, and
Bayesian Additive Regression Trees.

"""


class Bagging:
    
    def out_of_bag():
        """Out-of-Bag (OOB) error estimation"""
        pass

    
class RandomForest:
    
    def out_of_bag():
        """Out-of-Bag (OOB) error estimation"""
        pass


class Boosting:
    pass


class BayesianAdditiveRegressionTrees:
    pass




""" 
This module was created to handle Unsupervised 
Learning methods. 

"""


class PrincipalComponentsAnalysis:
    pass


class KMeans:
    """Numerical clustering
    """
    pass


class KMode:
    """Categorical clustering
    """
    pass


class KPrototype:
    """Numerical and Categorical clustering
    """
    pass


class HierarchicalClustering:
    pass


""" 
This module was created to handle some 
of the Shrinkage methods.

"""


def ridge_regression():
    pass


def the_lasso():
    pass


def selecting_tuning_parameter():
    """Get best tuning parameter for Ridge Regression 
    and Lasso procedures.
    
    Implementing ridge regression and 
    the lasso requires a method for selecting 
    a value for the tuning parameter λ.
    """
    pass


""" 
This module was created to handle multiple testing 
methods. 

"""


class Bonferroni:
    """Method to control Family-Wise Error Rate (FWER)
    """
    pass


class HolmStepDown:
    """Holm's method, also known as Holm's step-down procedure or the Holm-Bonferroni 
    method, is an alternative to the Bonferroni procedure. Holm's method 
    controls the FWER, but it is less conservative than Bonferroni, in 
    the sense that it will reject more null hypotheses, typically resulting in fewer 
    Type II errors and hence greater power.
    """
    pass


class BenjaminiHochberg:
    """Procedure to control the False Doscpvery Rate (FDR)
    """
    pass


class PValueResampling:
    pass


class FDRResampling:
    pass


""" 
This module was created to handle some 
of the Subset Selection methods.

"""


def best_subset_selection():
    pass


def forward_stepwise_selection():
    pass


def backward_stepwise_selection():
    pass


def hybrid_stepwise_selection():
    pass


def one_standard_error_rule():
    """We first calculate the standard error of the 
    estimated test MSE for each model size, and then select 
    the smallest model for which the estimated test error 
    is within one standard error of the lowest point on the curve.
    """
    pass


""" 
This module was created to handle some 
of the Test Error Estimation methods.

"""


def cp():
    """Mallow's Cp method.
    """
    pass


def aic():
    """Akaike information criterion (AIC).
    """
    pass


def bic():
    """Bayesian information criterion (BIC).
    """
    pass


def adjusted_r2():
    """Adjusted R^2.
    """
    pass


""" 
This module was created to create transformers
which will receive some DataFrame and return a transformed
DataFrame.

"""


from abc import ABC, abstractmethod
from pandas.core.frame import DataFrame
from drtools.logs import Log
import logging


class BasicTransformer(ABC):
    """Abstract class to handle the Datasets transformations
        
    """
    
    def __init__(
        self,
        LOGGER: Log=None
    ) -> None:
        self.LOGGER = logging if LOGGER is None else LOGGER
    
    @abstractmethod
    def transform(
        self, 
        dataframe: DataFrame
    ) -> DataFrame:
        pass


""" 
This module was created to handle non-linear
models such as Polynomial Regression, Regression 
Splines, Smoothing Splines, Local Regression 
and Generalized Additive Models (GAMs).

"""


class PolynomialRegression:
    pass

    
class RegressionSpline:
    
    def spline():
        pass
    
    def natural_spline():
        pass
    
    def choosing_the_number_and_locations_of_the_knots(self):
        pass
    
    
class SmoothingSpline:
    
    def choosing_the_smoothing_parameter():
        """The leave-one-out cross-validation error (LOOCV) 
        can be computed very eﬃciently for smoothing splines, 
        with essentially the same cost as computing a single fit.
        """
        pass
    

class LocalRegression:
    pass


class GeneralizedAdditiveModel:
    pass


class RegressionGAM(GeneralizedAdditiveModel):
    """Regression by Generalized Additive Model.
    """
    pass


class ClassificationGAM(GeneralizedAdditiveModel):    
    """Classification by Generalized Additive Model.
    """
    pass
